<copyright> Extension class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996-2001 Pieter J. Schoenmakers.

    This file is part of TOM.  TOM is distributed under the terms of the
    TOM License, a copy of which can be found in the TOM distribution; see
    the file LICENSE.

    <id>$Id: Extension.t,v 1.12 2001/04/08 22:14:43 bruce Exp $</id>
    </copyright>

<doc> The {Extension} class represents the runtime structures for class extensions.
    All classes have at least one extension, the main extension, which defines
    the behaviors and the state for that class.  All other extensions are defined by
    the programmer as extensions to the class.  </doc>

implementation class
Extension: State, Conditions
{
  <doc> All extensions, here to be protected against gc.  </doc>
  static MutableArray extensions;
}

<doc> Designated allocator.  Do not use {alloc} or plain {new}.  </doc>
instance (id)
  new pointer p
{
  if (!extensions)
    extensions = [MutableObjectArray new];

  Extension r = [[self alloc] init p];
  [extensions add r];

  = r;
}

end;

implementation instance
Extension
{
  <doc> The runtime structure describing this extension.  </doc>
  pointer rti;

  <doc> The selectors for this extension's methods.  </doc>
  Indexed method_selectors;

  <doc> The names of this extension's variables.  </doc>
  Indexed var_names;
}

<doc> An {Extension} should never be deallocated.  This method raises,
    which is a panic during garbage collection...  </doc>
void
  dealloc
{
  [[Condition for self class program-condition
	      message "dealloc considered harmful"] raise];
}

<doc> Return {YES} if this extension provides an implementation for the
    selector {sel}.  </doc>
extern boolean
  implements selector sel;

<doc> Designated initializer.  </doc>
protected id (self)
  init pointer r
{
  rti = r;
}

<doc> Return the class object to which this extension belongs.  </doc>
extern class (State)
  meta;

<doc> Return the name of this extension.  This will be {nil} for the main
    extension of a class.  </doc>
extern String
  name;

<doc> Return the selectors for the methods in this extension.  </doc>
extern Indexed
  methods;

<doc> Return whether or not this extension defines variable additions. </doc>
extern boolean
  hasState;

<doc> Return the names of the variables in this extension.  </doc>
extern Indexed
  variables;

<doc> Return the type of the variable named {name}.  The return value will
    correspond with one of the {TYPEDESC_*} constants defined on the
    {Constants} class.  </doc>
extern int
  typeOfVariableNamed String name
		   in All object;

extern Any
  valueOfVariableNamed String name
		    in All object;

extern void
	 setValue dynamic value
  ofVariableNamed ByteString name
	       in All object;

OutputStream
  writeFields OutputStream s
{
  = [s print (" name=", [self name], " class=", [self meta])];
}

/******************** perform ********************/

<doc> The equivalent of {perform with} where the method invoked is defined
    by this extension instead of the receiving object.  Obviously, the
    {object} should actually have this extension as one of its extensions,
    i.e. {[object isKindOf [self meta]]} should be a precondition (and a
    postcondition too, but we're not interested after the fact).  </doc>
extern dynamic
  perform selector sel
       on All object
     with dynamic arguments;

extern dynamic
  perform selector sel
       on All object
	: Array arguments = nil;

end;
